import json
import logging
import os
from random import sample
from string import ascii_letters, digits

import requests
from django.http import JsonResponse, HttpResponseRedirect, HttpResponse
from rest_framework.views import APIView
from wechatpy import WeChatPay
from lxml import etree as et
from rest_framework import status

class WeChatPayment(APIView):
    # 通过小程序前端 wx.login() 接口获取临时登录凭证 code
    # 将 code 作为参数传入，调用 get_user_info() 方法获取 openid
    #
    def get_user_info(self):
        # https://open.weixin.qq.com/connect/oauth2/authorize?appid=wx0c2a8db23b2e7c28&redirect_uri=REDIRECT_URI&response_type=code&scope=snsapi_userinfo&state=STATE#wechat_redirect
        """
         接口测试
         :param request:
         :return:
         """
        code = self.GET.get('code', 0)  # 保留俩位小数  前端传回的金额数据
        """
        使用 临时登录凭证code 获取 session_key 和 openid 等
        支付部分仅需 openid，如需其他用户信息请按微信官方开发文档自行解密
         """
        req_params = {
            'appid': settings.WECHAT['APPID'],
            'secret': settings.WECHAT['APPSECRET'],
            'js_code': code,
            'grant_type': 'authorization_code',
        }
        user_info = requests.get('https://api.weixin.qq.com/sns/jscode2session', params=req_params, timeout=3,
                                 verify=False)
        return user_info.json()

    def pay(self):
        # code = self.GET.get("code", None)
        # openid = self.get_user_info(code)['openid']

        pay = WeChatPay(settings.WECHAT['APPID'], settings.WECHAT['MERCHANT_KEY'], settings.WECHAT['MCH_ID'])
        order = pay.order.create(
            trade_type=settings.WECHAT['TRADE_TYPE'],  # 交易类型，小程序取值：JSAPI
            body=settings.WECHAT['BODY'],  # 商品描述，商品简单描述
            total_fee=settings.WECHAT['TOTAL_FEE'],  # 标价金额，订单总金额，单位为分
            notify_url=settings.WECHAT['NOTIFY_URL'],  # 通知地址，异步接收微信支付结果通知的回调地址，通知url必须为外网可访问的url，不能携带参数。
            user_id="osG795T10McjYnTD36k_Jh7JSCNg"  # 用户标识，trade_type=JSAPI，此参数必传，用户在商户appid下的唯一标识。
        )
        wxpay_params = pay.jsapi.get_jsapi_params(order['prepay_id'])

        return JsonResponse(wxpay_params)

    def callback(self):
        _xml = self.body
        # 拿到微信发送的xml请求 即微信支付后的回调内容
        xml = str(_xml, encoding="utf-8")
        print("xml", xml)
        return_dict = {}
        tree = et.fromstring(xml)
        # xml 解析
        return_code = tree.find("return_code").text
        try:
            if return_code == 'FAIL':
                # 官方发出错误
                return_dict['message'] = '支付失败'
                # return Response(return_dict, status=status.HTTP_400_BAD_REQUEST)
            elif return_code == 'SUCCESS':
                # 拿到自己这次支付的 out_trade_no
                _out_trade_no = tree.find("out_trade_no").text
                # TODO 这里省略了 拿到订单号后的操作 看自己的业务需求
        except Exception as e:
            pass
        finally:
            return HttpResponse(return_dict, status=status.HTTP_200_OK)


from django.http import JsonResponse
from rest_framework.views import APIView

from xj_payment.services.payment_service import PaymentService
from ..utils.model_handle import parse_data


class Payment(APIView):
    def pay(self):
        params = parse_data(self)
        payment = PaymentService.pay(params)
        return JsonResponse({
            'err': 0,
            'msg': 'OK',
            "data": payment
        })

    def refund(self):
        params = parse_data(self)
        payment = PaymentService.refund(params)
        return JsonResponse({
            'err': 0,
            'msg': 'OK',
            "data": payment
        })



# 应用名称
from django.urls import re_path

from xj_payment.apis import alipay, transaction_inquiry, wechat, payment
from .apis import payment_alipay
from .apis import payment_wechat
from .apis import payment_unionpay
from .apis import payment
from .apis import wechat

app_name = 'payment'

urlpatterns = [
    # re_path(r'^alipay_payment/?$', alipay.Payment.as_view(), ),
    # re_path(r'^alipay_payment/?$', alipay.Payment.test)
    # re_path(r'^get_pay_url/?$', alipay.Payment.get_pay_url),  # 获取支付宝支付链接
    re_path(r'^get_pay_url/?$', payment_alipay.PaymentAlipay.get_pay_url),  # 获取支付宝支付链接
    # re_path(r'^get_result/?$', alipay.Payment.pay_result),  # 支付宝处理完成后同步回调通知
    re_path(r'^get_result/?$', payment_alipay.PaymentAlipay.pay_result),  # 支付宝处理完成后同步回调通知
    re_path(r'^update_order/?$', alipay.Payment.update_order),  # 支付宝处理完成后支付宝服务器异步回调通知
    # re_path(r'^refund/?$', alipay.Payment.refund),  # 支付宝退款
    re_path(r'^close/?$', alipay.Payment.close),  # 关闭订单

    re_path(r'^refund_inquiry/?$', transaction_inquiry.query.refund_inquiry),  # 支付宝退款查询
    re_path(r'^trade_query/?$', transaction_inquiry.query.trade_query),  # 支付宝下单查询

    re_path(r'^get_user_info/?$', payment_wechat.PaymentWechat.get_user_info),  # 获取用户标识接口
    re_path(r'^applets_pay/?$', payment_wechat.PaymentWechat.payment_applets_pay),  # 小程序支付接口
    re_path(r'^wechat_py/?$', wechat.WeChatPayment.pay),  # 微信支付接口
    # re_path(r'^wechat_pay/?$', payment.WechatPayDAL.create_order_info),  # 微信支付接口
    re_path(r'^scan_pay/?$', payment_wechat.PaymentWechat.payment_scan_pay),  # 微信扫码支付
    re_path(r'^red_envelopes/?$', payment_wechat.PaymentWechat.payment_red_envelopes),  # 微信红包
    re_path(r'^wechat_callback/?$', payment_wechat.PaymentWechat.callback),  # 回调接口
    re_path(r'^wechat_callback_v3/?$', payment_wechat.PaymentWechat.callback_v3),  # 回调接口v3

    re_path(r'^unipay/?$', payment_unionpay.PaymentUnionPay.unipay),

    re_path(r'^pay/?$', payment.Payment.pay),

    re_path(r'^refund/?$', payment.Payment.refund),

    # re_path(r'^pay/?$', wechat_payment.WeChatPayment.pay),
    # re_path(r'^pay/?$', wechat.WeChatPayment.pay),
    # 支付结果回调
    # re_path(r'^payNotify/', views.WeChatPayNotifyViewSet.as_view(), name='pay_notify'),

]



import json
import os
import time
from random import sample
from string import ascii_letters, digits
import logging
from pathlib import Path

from celery.utils.serialization import jsonify

from main.settings import BASE_DIR
from ..utils.j_config import JConfig
from ..utils.j_dict import JDict

from django.http import JsonResponse, HttpResponseRedirect, HttpResponse
from rest_framework.views import APIView
from wechatpayv3 import SignType, WeChatPay, WeChatPayType
from ..services.payment_wechat_service import PaymentWechatService

module_root = str(Path(__file__).resolve().parent)
# 配置之对象
main_config_dict = JDict(JConfig.get_section(path=str(BASE_DIR) + "/config.ini", section="xj_payment"))
module_config_dict = JDict(JConfig.get_section(path=str(BASE_DIR) + "/config.ini", section="xj_payment"))

app_id = main_config_dict.wechat_service_app_id or module_config_dict.wechat_service_app_id or ""

app_secret = main_config_dict.wechat_service_app_secret or module_config_dict.wechat_service_app_secret or ""

mch_id = main_config_dict.wechat_service_mch_id or module_config_dict.wechat_service_mch_id or ""

merchant_key = main_config_dict.wechat_service_merchant_key or module_config_dict.wechat_service_merchant_key or ""

sub_appid = main_config_dict.wechat_merchant_app_id or module_config_dict.wechat_merchant_app_id or ""

sub_app_secret = main_config_dict.wechat_merchant_app_secret or module_config_dict.wechat_merchant_app_secret or ""

sub_mch_id = main_config_dict.wechat_merchant_mch_id or module_config_dict.wechat_merchant_mch_id or ""

apiv3_secret = main_config_dict.wechat_apiv3_secret or module_config_dict.wechat_apiv3_secret or ""

trade_type = main_config_dict.wechat_trade_type or module_config_dict.wechat_trade_type or ""

cert_serial_no = main_config_dict.wechat_cert_serial_no or module_config_dict.wechat_cert_serial_no or ""

cert_dir = main_config_dict.wechat_cert_dir or module_config_dict.wechat_cert_dir or ""
# 交易类型，小程序取值：JSAPI

# 商品描述，商品简单描述
description = main_config_dict.wechat_body or module_config_dict.wechat_body or ""
# 标价金额，订单总金额，单位为分
total_fee = main_config_dict.wechat_total_fee or module_config_dict.wechat_total_fee or ""
# 通知地址，异步接收微信支付结果通知的回调地址，通知url必须为外网可访问的url，不能携带参数。
notify_url = main_config_dict.wechat_notify_url or module_config_dict.wechat_notify_url or ""

private_key_path = main_config_dict.wechat_merchant_private_key_file or module_config_dict.wechat_merchant_private_key_file or (
        str(BASE_DIR) + "/config/apiclient_key.pem")
# 读取文件获取密钥
private_key = open(private_key_path, encoding="utf-8").read() if os.path.exists(private_key_path) else ""

# 微信支付商户号（直连模式）或服务商商户号（服务商模式，即sp_mchid)
MCHID = mch_id

# 商户证书私钥
# with open('path_to_key/apiclient_key.pem') as f:
#     PRIVATE_KEY = f.read()

# 商户证书序列号
CERT_SERIAL_NO = cert_serial_no

# API v3密钥， https://pay.weixin.qq.com/wiki/doc/apiv3/wechatpay/wechatpay3_2.shtml
APIV3_KEY = apiv3_secret

# APPID，应用ID或服务商模式下的sp_appid
APPID = app_id

# 回调地址，也可以在调用接口的时候覆盖
NOTIFY_URL = notify_url

# 微信支付平台证书缓存目录，减少证书下载调用次数
# 初始调试时可不设置，调试通过后再设置，示例值:'./cert'
CERT_DIR = cert_dir

# 日志记录器，记录web请求和回调细节
logging.basicConfig(filename=os.path.join(os.getcwd(), 'demo.log'), level=logging.DEBUG, filemode='a',
                    format='%(asctime)s - %(process)s - %(levelname)s: %(message)s')
LOGGER = logging.getLogger("demo")

# 接入模式:False=直连商户模式，True=服务商模式
PARTNER_MODE = True

# 代理设置，None或者{"https": "http://10.10.1.10:1080"}，详细格式参见https://docs.python-requests.org/zh_CN/latest/user/advanced.html
PROXY = None


class WeChatPayment(APIView):
    def pay(self):
        # jsapi下单，wxpay初始化的时候，wechatpay_type设置为WeChatPayType.JSAPI。
        # 下单成功后，将prepay_id和其他必须的参数组合传递给JSSDK的wx.chooseWXPay接口唤起支付
        wxpay = WeChatPay(
            wechatpay_type=WeChatPayType.JSAPI,
            mchid=mch_id,
            private_key=private_key,
            cert_serial_no=CERT_SERIAL_NO,
            apiv3_key=APIV3_KEY,
            appid=app_id,
            notify_url=NOTIFY_URL,
            cert_dir=CERT_DIR,
            logger=LOGGER,
            partner_mode=PARTNER_MODE,
            proxy=PROXY)

        out_trade_no = ''.join(sample(ascii_letters + digits, 8))
        description = 'demo-description'
        amount = 1
        # payer = {'openid': 'oj8KR5AR0sV20j96VBcnWulf7bbs'}
        # payer = json.dumps(payer)
        code, message = wxpay.pay(
            description=description,
            out_trade_no=out_trade_no,
            sub_appid=sub_appid,
            sub_mchid=sub_mch_id,
            amount={'total': amount},
            payer={'sub_openid': 'oj8KR5AR0sV20j96VBcnWulf7bbs'}
        )
        result = json.loads(message)
        if code in range(200, 300):
            prepay_id = result.get('prepay_id')
            timestamp = PaymentWechatService.to_text(int(time.time()))
            noncestr = PaymentWechatService.random_string(32)
            package = 'prepay_id=' + prepay_id
            paysign = wxpay.sign([sub_appid, timestamp, noncestr, package])
            signtype = 'RSA'
            return JsonResponse({'code': 0, 'result': {
                'appId': sub_appid,
                'timeStamp': timestamp,
                'nonceStr': noncestr,
                'package': 'prepay_id=%s' % prepay_id,
                'signType': signtype,
                'paySign': paysign
            }})
        else:
            return JsonResponse({'code': -1, 'result': {'reason': result.get('code')}})



#!/usr/bin/python3
from __future__ import absolute_import
import base64
from cryptography.hazmat.primitives.ciphers.aead import AESGCM
from Cryptodome.PublicKey import RSA
from Cryptodome.Signature import pkcs1_15
from Cryptodome.Hash import SHA256
import hashlib
import random
import time
from urllib3.util import parse_url


class WechatPayDALBase(object):
    def __init__(self, mch_appid, mchid, v3key, serial_no, client_key):
        self.mch_appid = mch_appid
        self.mchid = mchid
        self.v3key = v3key
        # serial_no可通过openssl直接获取, 例: openssl x509 -in 1900009191_20180326_cert.pem -noout -serial
        self.serial_no = serial_no

        with open(client_key, 'r') as ifile:
            pkey = RSA.importKey(ifile.read())
        self.signer = pkcs1_15.new(pkey)

    def compute_sign_v3(self, method, url, body):
        '''
        V3签名逻辑
        '''
        ts = int(time.time())
        nonce = self.generate_nonce()
        uparts = parse_url(url)
        ustr = uparts.path + ('?{}'.format(uparts.query) if uparts.query else '')
        content = '{}\n{}\n{}\n{}\n{}\n'.format(method, ustr, ts, nonce, body)

        digest = SHA256.new(content.encode('utf-8'))
        sign_v = base64.b64encode(self.signer.sign(digest)).decode('utf-8')
        sign_str = 'serial_no="{}",mchid="{}",timestamp="{}",nonce_str="{}",signature="{}"'.format(
            self.serial_no, self.mchid, ts, nonce, sign_v)
        return sign_str

    def get_pay_sign_info(self, prepay_id):
        ts = int(time.time())
        nonce = self.generate_nonce()
        content = '{}\n{}\n{}\n{}\n'.format(self.mch_appid, ts, nonce, prepay_id)

        digest = SHA256.new(content.encode('utf-8'))
        sign_v = base64.b64encode(self.signer.sign(digest)).decode('utf-8')
        return {
            'appId': self.mch_appid,
            # 'partnerid': self.mchid,
            'timeStamp': str(ts),
            'nonceStr': nonce,
            "signType": "RSA",
            # 'prepay_id': prepay_id,
            # 'package': 'Sign=WXPay',
            'package': 'prepay_id=' + prepay_id,
            'sign': sign_v,
        }

    def decrypt_v3(self, dct):
        key_bytes = self.v3key.encode()
        nonce_bytes = dct['encrypt_certificate']['nonce'].encode()
        ad_bytes = dct['encrypt_certificate']['associated_data'].encode()
        data = base64.b64decode(dct['encrypt_certificate']['ciphertext'])

        aesgcm = AESGCM(key_bytes)
        ret = aesgcm.decrypt(nonce_bytes, data, ad_bytes)
        return ret

    def make_headers_v3(self, url, headers=None, body='', method='GET'):
        '''
        微信支付V3版本签名header生成函数
        '''
        if not headers:
            headers = {}
        headers['Accept'] = 'application/json'
        sign = self.compute_sign_v3(method, url, body)
        auth_info = 'WECHATPAY2-SHA256-RSA2048 {}'.format(sign)
        headers['Authorization'] = auth_info
        return headers

    def generate_nonce(self):
        rnd = int(time.time()) + random.randint(100000, 1000000)
        nonce = hashlib.md5(str(rnd).encode()).hexdigest()[:16]
        return nonce

    def generate_partner_trade_no(self, ndt):
        # 商户订单号必须保证唯一
        # 支付失败、异常一定要使用原订单号重试，更换订单号可能造成重复支付
        # 合理判断逻辑: 应先判断协议字段返回，再判断业务返回，最后判断交易状态
        rpart = random.randint(100000, 1000000)
        tid = '{}{}'.format(ndt.strftime('WX%Y%m%d%H%M%S%f'), rpart)
        return tid
